Package: tfevents
Title: Write Events for 'TensorBoard'
Version: 0.0.5
Authors@R: c(
    person("Daniel", "Falbel", email = "dfalbel@gmail.com", role = c("aut", "cph")),
    person("Tomasz", "Kalinowski", email = "tomasz@posit.co", role = c("cre")),
    person(family = "Posit, PBC", role = c("cph")),
    person(family = "The tl::optional authors", role = c("cph"), comment = "For the vendored tl::optional code."),
    person("Mark", "Adler", role = c("cph"), comment = "For the included crc32c code.")
    )
Description: Provides a convenient way to log scalars, images, audio, and histograms in the 'tfevent' record file format. 
  Logged data can be visualized on the fly using 'TensorBoard', a web based tool that focuses on visualizing the training 
  progress of machine learning models.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
LinkingTo: Rcpp
Imports: Rcpp, withr, fs, rlang (>= 1.0.0), vctrs, blob, png, digest,
        cli, zeallot
Suggests: testthat (>= 3.0.0), tibble, tidyr, reticulate, rmarkdown,
        ggplot2, tensorflow, wav
SystemRequirements: libprotobuf, protobuf-compiler
URL: https://github.com/mlverse/tfevents,
        https://mlverse.github.io/tfevents/
BugReports: https://github.com/mlverse/tfevents/issues
Config/testthat/edition: 3
Config/Needs/website: tensorflow, webshot2, keras
NeedsCompilation: yes
Packaged: 2026-04-27 11:40:43 UTC; dfalbel
Author: Daniel Falbel [aut, cph],
  Tomasz Kalinowski [cre],
  Posit, PBC [cph],
  The tl::optional authors [cph] (For the vendored tl::optional code.),
  Mark Adler [cph] (For the included crc32c code.)
Maintainer: Tomasz Kalinowski <tomasz@posit.co>
Repository: CRAN
Date/Publication: 2026-04-27 12:40:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-27 21:19:21 UTC; unix
Archs: tfevents.so.dSYM
