# Generated by roxygen2: do not edit by hand

S3method(as_event,character)
S3method(as_event,list)
S3method(as_event,numeric)
S3method(as_event,tfevents_summary_values)
S3method(as_tensor_proto,array)
S3method(as_tensor_proto,blob)
S3method(as_tensor_proto,character)
S3method(as_tensor_proto,list)
S3method(format,tfevents_event)
S3method(format,tfevents_summary)
S3method(format,tfevents_summary_values)
S3method(summary_audio,array)
S3method(summary_audio,blob)
S3method(summary_audio,raw)
S3method(summary_histogram,array)
S3method(summary_histogram,numeric)
S3method(summary_image,array)
S3method(summary_image,blob)
S3method(summary_image,ggplot)
S3method(summary_image,raw)
S3method(summary_text,character)
S3method(value,tfevents_summary_values)
S3method(value,tfevents_summary_values_audio)
S3method(value,tfevents_summary_values_histograms)
S3method(value,tfevents_summary_values_images)
S3method(value,tfevents_summary_values_scalars)
S3method(value,tfevents_summary_values_text)
S3method(vec_cast,hparams_experiment.hparams_experiment)
S3method(vec_cast,hparams_hparam_info.hparams_hparam_info)
S3method(vec_cast,hparams_interval.hparams_interval)
S3method(vec_cast,hparams_session_start_info.hparams_session_start_info)
S3method(vec_cast,list.hparams_hparams_plugin_data)
S3method(vec_cast,summary_summary_image.summary_summary_image)
S3method(vec_cast,tensor_proto.tensor_proto)
S3method(vec_cast,tfevents_event.tfevents_event)
S3method(vec_cast,tfevents_summary.tfevents_summary_values)
S3method(vec_cast,tfevents_summary_values.tfevents_summary_values)
S3method(vec_ptype2,hparams_experiment.hparams_experiment)
S3method(vec_ptype2,hparams_hparam_info.hparams_hparam_info)
S3method(vec_ptype2,hparams_hparams_plugin_data.list)
S3method(vec_ptype2,hparams_interval.hparams_interval)
S3method(vec_ptype2,hparams_session_start_info.hparams_session_start_info)
S3method(vec_ptype2,summary_hparams_config.tfevents_summary_values)
S3method(vec_ptype2,summary_summary_image.summary_summary_image)
S3method(vec_ptype2,tfevents_event.tfevents_event)
S3method(vec_ptype2,tfevents_summary.tfevents_summary_values)
S3method(vec_ptype2,tfevents_summary_scalar.tfevents_summary_values)
S3method(vec_ptype2,tfevents_summary_values.summary_hparams_config)
S3method(vec_ptype2,tfevents_summary_values.tfevents_summary)
S3method(vec_ptype2,tfevents_summary_values.tfevents_summary_scalar)
S3method(vec_ptype2,tfevents_summary_values.tfevents_summary_values)
export(as_event)
export(collect_events)
export(events_logdir)
export(get_default_logdir)
export(get_global_step)
export(hparams_hparam)
export(hparams_metric)
export(local_logdir)
export(log_event)
export(log_hparams)
export(log_hparams_config)
export(set_default_logdir)
export(set_global_step)
export(summary_audio)
export(summary_histogram)
export(summary_hparams)
export(summary_hparams_config)
export(summary_image)
export(summary_metadata)
export(summary_scalar)
export(summary_text)
export(value)
export(with_logdir)
import(vctrs)
importFrom(Rcpp,sourceCpp)
importFrom(blob,blob)
importFrom(utils,tail)
importFrom(zeallot,"%<-%")
useDynLib(tfevents, .registration = TRUE)
