# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

create_event_file_iterator <- function(path, run_name) {
    .Call(`_tfevents_create_event_file_iterator`, path, run_name)
}

event_file_iterator_next <- function(iter) {
    .Call(`_tfevents_event_file_iterator_next`, iter)
}

get_wall_time <- function() {
    .Call(`_tfevents_get_wall_time`)
}

event_writer <- function(file) {
    .Call(`_tfevents_event_writer`, file)
}

write_events <- function(events, writers) {
    .Call(`_tfevents_write_events`, events, writers)
}

